package price

import (
	"bytes"
	"fmt"
	"log"
	"strconv"
	"time"

	"git.paulbsd.com/paulbsd/fuelprices/src/config"
	"git.paulbsd.com/paulbsd/fuelprices/src/xmlfile"
	"github.com/antchfx/xmlquery"
	client "github.com/influxdata/influxdb1-client/v2"
)

// GetPrices parses the XML file and get values of prices
func GetPrices(c *config.Config, prices *[]Price, xmlfile *xmlfile.XMLFile) (err error) {
	var xml *xmlquery.Node
	var valueattr = "valeur"

	file := bytes.NewReader(xmlfile.Content)
	xml, err = xmlquery.Parse(file)
	if err != nil {
		return
	}

	for _, station := range c.Pos {
		for _, fuel := range c.Types {

			query := fmt.Sprintf(c.XPathBase, station, fuel)
			list := xmlquery.FindOne(xml, query)

			if list != nil {
				for _, i := range list.Attr {
					if i.Name.Local == valueattr {
						if s, err := strconv.ParseFloat(i.Value, 64); err == nil {
							*prices = append(*prices, Price{ID: station, Fuel: fuel, Amount: s})
						}
					}
				}
			} else {
				log.Println(fmt.Sprintf("Fuel type not found for point of sale, skipping. Query : %s", query))
			}
		}
	}
	return
}

// SendToInflux sends time series data to influxdb
func SendToInflux(c *config.Config, prices *[]Price) (err error) {
	httpClient, err := client.NewHTTPClient(client.HTTPConfig{
		Addr:     c.InfluxURL,
		Username: c.InfluxUser,
		Password: c.InfluxPass,
	})
	if err != nil {
		return
	}

	defer httpClient.Close()

	bp, err := client.NewBatchPoints(client.BatchPointsConfig{
		Database: c.InfluxDB,
	})
	if err != nil {
		return
	}

	for _, p := range *prices {

		tags := map[string]string{"pdv": p.ID, "fuel": p.Fuel}
		fields := map[string]interface{}{"value": p.Amount}

		point, _ := client.NewPoint(
			c.Table,
			tags,
			fields,
			time.Now(),
		)

		log.Println(point)

		bp.AddPoint(point)
		err = httpClient.Write(bp)
		if err != nil {
			return
		}
	}

	return nil
}

// Price contains price of points of sale
type Price struct {
	ID     string
	Fuel   string
	Amount float64
}
